package lu.tudor.santec.dicom.exporter;

import ij.ImageJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.ImageWindow;
import ij.io.FileInfo;
import ij.io.ImageWriter;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Properties;

import lu.tudor.santec.dicom.gui.header.DicomHeader;
import lu.tudor.santec.dicom.gui.header.HeaderTag;

import org.dcm4che2.data.BasicDicomObject;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.data.Tag;
import org.dcm4che2.data.UID;
import org.dcm4che2.data.VR;
import org.dcm4che2.io.DicomOutputStream;
import org.dcm4che2.iod.value.ImageTypeValue1;
import org.dcm4che2.iod.value.ImageTypeValue2;
import org.dcm4che2.util.UIDUtils;

public class DicomExporter {

    private String charset = "ISO_IR 100";
    
    private String transferSyntax = UID.ExplicitVRBigEndian;
    
    private Properties cfg = new Properties();

    public DicomExporter() {
        try {
            cfg.load(DicomExporter.class.getResourceAsStream("DicomExporter.cfg"));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    public void writeImage(DicomObject header, ImagePlus ip, File dcmFile) throws IOException {
	
	
            FileOutputStream fos = new FileOutputStream(dcmFile);
            BufferedOutputStream bos = new BufferedOutputStream(fos);
            DicomOutputStream dos = new DicomOutputStream(bos);
            
            FileInfo fi = ip.getFileInfo();
            Object pixels = fi.pixels;
            ImageWriter iw = new ImageWriter(fi);
            try {
                dos.writeDicomFile(header);
                int pixelLength = 0;
                if (ip.getBitDepth() == 8) {
                    pixelLength = ((byte[])pixels).length;
                    dos.writeHeader(Tag.PixelData, VR.OB, pixelLength);
                } else if (ip.getBitDepth() == 16) {
                    pixelLength = ((short[])pixels).length * 2;
                    dos.writeHeader(Tag.PixelData, VR.OW, pixelLength);
                } else if (ip.getBitDepth() == 24) {
                    pixelLength = ((int[])pixels).length * 3;
                    dos.writeHeader(Tag.PixelData, VR.OW, pixelLength);
                }
                 
                
                // write the pixel data
                iw.write(dos);

                if ((pixelLength&1) != 0) {
                    dos.write(0);
                }
                
//                dos.writeHeader(Tag.SequenceDelimitationItem, null, 0);
                
            } catch (Exception e ) {
        	e.printStackTrace();
            } finally {
                dos.close();
            }
    }    

    @SuppressWarnings("unchecked")
    public DicomObject createHeader(ImagePlus ip, boolean derived, boolean keepUID, boolean keepSeriesUID) throws IOException {

	DicomObject header = new BasicDicomObject();
	header.putString(Tag.SpecificCharacterSet, VR.CS, charset);

	DicomHeader origHeader= (DicomHeader) ip.getProperty(DicomHeader.class.getSimpleName());
	
	if (origHeader != null && origHeader.getDicomObject() != null) {
	    System.out.println("using original Header");
	    header = origHeader.getDicomObject();
	} else {
	    DicomHeader dh = new DicomHeader(ip);
	    if (dh.isEmpty()) {
		// set default attributes
		for (Enumeration en = cfg.propertyNames(); en.hasMoreElements();) {
		    String key = (String) en.nextElement();
		    int[] tagPath = Tag.toTagPath(key);
		    int last = tagPath.length - 1;
		    VR vr = header.vrOf(tagPath[last]);
		    if (vr == VR.SQ) {
			header.putSequence(tagPath);
		    } else {
			System.out.println("adding header field: "  + key + " = " + cfg.getProperty(key));
			try {
			    header.putString(tagPath, vr, cfg.getProperty(key));
			} catch (Exception e) {
			    e.printStackTrace();
			}
		    }
		}
	    } else {
		for (HeaderTag ht : dh.headerTags) {
		    String key = (String) ht.tagNr.replaceAll(",", "");
		    int tag = Tag.toTag(key);
		    try {
			header.putString(tag, null, ht.tagValue);
		    } catch (Exception e) {
		    }
		}
	    }
	}
	
	

	// set attributes from the image to the header
	int bitDepth = ip.getBitDepth();
	if (bitDepth == 8) {
	    header.putString(Tag.PhotometricInterpretation, VR.CS,"MONOCHROME");
	    header.putInt(Tag.SamplesPerPixel, VR.US, 1);
	    header.putInt(Tag.BitsAllocated, VR.US, 8);
	    header.putInt(Tag.HighBit, VR.US, 7);
	} else if (bitDepth == 16) {
	    header.putString(Tag.PhotometricInterpretation, VR.CS,"MONOCHROME2");
	    header.putInt(Tag.SamplesPerPixel, VR.US, 1);
	    header.putInt(Tag.BitsAllocated, VR.US, 16);
	    header.putInt(Tag.HighBit, VR.US, 15);
	} else if (bitDepth == 24) {
	    header.putString(Tag.PhotometricInterpretation, VR.CS, "RGB");
	    header.putInt(Tag.SamplesPerPixel, VR.US, 3);
	    header.putInt(Tag.BitsAllocated, VR.US, 24);
	    header.putInt(Tag.HighBit, VR.US, 23);
	}
	header.putInt(Tag.BitsStored, VR.US, bitDepth);
	header.putInt(Tag.Rows, VR.US, ip.getHeight());
	header.putInt(Tag.Columns, VR.US, ip.getWidth());
	header.putInt(Tag.PixelRepresentation, VR.US, 0);

	ensureUS(header, Tag.BitsAllocated, 8);
	ensureUS(header, Tag.BitsStored, header.getInt(Tag.BitsAllocated));
	ensureUS(header, Tag.HighBit, header.getInt(Tag.BitsStored) - 1);
	ensureUS(header, Tag.PixelRepresentation, 0);
	
	ensureUID(header, Tag.StudyInstanceUID);

	if (keepSeriesUID) {
	    ensureUID(header, Tag.SeriesInstanceUID);
	} else {
	    createUID(header, Tag.SeriesInstanceUID);	    	    
	}
	
	if (keepUID) {
	    ensureUID(header, Tag.SOPInstanceUID);
	} else {
	    createUID(header, Tag.SOPInstanceUID);
	}

	
	Date now = new Date();
	ensureDate(header, Tag.InstanceCreationDate, VR.DA, now);
	ensureDate(header, Tag.InstanceCreationTime, VR.TM, now);
	
	// set ImageType to derived Secondary if wanted
	if (derived) {
	    header.putString(Tag.ImageType, VR.CS, ImageTypeValue1.DERIVED + "/" + ImageTypeValue2.SECONDARY);
	}
	    
	    
	header.initFileMetaInformation(transferSyntax);

	return header;
    }
    
    private void createUID(DicomObject attrs, int tag) {
         attrs.putString(tag, VR.UI, UIDUtils.createUID());   
    }

    private void ensureUID(DicomObject attrs, int tag) {
        if (!attrs.containsValue(tag)) {
            attrs.putString(tag, VR.UI, UIDUtils.createUID());
        }        
    }

    private void ensureUS(DicomObject attrs, int tag, int val) {
        if (!attrs.containsValue(tag)) {
            attrs.putInt(tag, VR.US, val);
        }        
    }    
    
    private void ensureDate(DicomObject attrs, int tag, VR vr , Date val) {
        if (!attrs.containsValue(tag)) {
            attrs.putDate(tag, vr, val);
        }        
    }    
    
    @SuppressWarnings("deprecation")
    public static void main(String[] args) {
        try {
            
            DicomExporter exporter = new DicomExporter();
            
            File dcmFile = new File("/home/hermenj/jhermen.dcm");
            
//            ImagePlus ip = new ImagePlus("/home/hermenj/16bit.tiff");
//            ImagePlus ip = new ImagePlus("/home/hermenj/8bit.gif");
            ImagePlus ip = new ImagePlus("/home/hermenj/im0");
            
            DicomObject dObj = exporter.createHeader(ip, true, true, true);
            
            exporter.writeImage(dObj, ip, dcmFile);
            
//            DicomHeader dh = new DicomHeader(dcmFile);
//            System.out.println(dh);
            
//            IJ.debugMode = true;
            
            
            
            ImagePlus image = new ImagePlus(dcmFile.getAbsolutePath());
           new ImageJ().show();
           WindowManager.addWindow(new ImageWindow(image));
            
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
   
}
